#!/usr/bin/perl -w 

use strict;
use warnings;
use v5.10;
use lib 'lib', '../lib'; # able to run prove in project dir and .t locally
    
use Test::More tests => 2;

use_ok('Music::AirGuitar');
use_ok('Music::AirGuitar::Performance');

exit 0;
